% $Id: BRel.oz,v 1.1 2011/10/06 01:20:06 leavens Exp leavens $
% You can use AppendMap if you want; if you don't, comment out the next line.
\insert 'AppendMap.oz'
%% We are representing binary relations as association lists with
%% #-pairs of the form <S>#<T>, without duplicate pairs.
%% (Pairs are duplicates if they are == to each other.)
%% <BRel S T> ::= nil
%%         |   <#-Pair S T>'|'<BRel S T>
%% <#-Pair S T> ::= <S>#<T>
%%
%% We sometimes refer to the first element of a pair as a "key"
%% and the second element as the "value" associated with that key.
%% We sometimes also think of a BRel as a set of #-pairs
%% and use set comprehension notation to explain functions.

%%-------------------------------------------------------
%% Start of provided functions section.
%% Do not change any of the following provided functions.
%% The code you are to write follows this section.
%%-------------------------------------------------------
declare
fun {EmptyBRel}
   %% Return the empty binary relation
   nil
end

fun {BRelHas BRel Key Val}
   %% Return true just when BRel associates Key to Val (has Key#Val as a #-pair)
   {Some BRel fun {$ K#V} K == Key andthen V == Val end}
end

fun {BRelHasKey BRel Key}
   %% Return true just when there is a #-pair in BRel whose first element is Key
   {Some BRel fun {$ K#_} K == Key end}
end

fun {IsBRel Val}
   %% Return true just when BRel is a legal binary relation
   case Val of
      nil then true
   [] (K#V)|Rest then {Not {BRelHas Rest K V}} andthen {IsBRel Rest}
   end
end

fun {BRelSubset BRel1 BR2}
   %% Return true just when BRel1 is a subset of BR2
   case BRel1 of
      nil then true
   [] (K#V)|Rest then {BRelHas BR2 K V} andthen {BRelSubset Rest BR2}
   end
end

fun {BRelEqual BRel1 BR2}
   %% Return true just when BRel1 is the same relation as BR2.
   {BRelSubset BRel1 BR2} andthen {BRelSubset BR2 BRel1}
end

fun {BRelAdd BRel K V}
   %% Return the binary relation that has all associations in BRel 
   %% and that also associates K to V.
   %% That is, the result is {K#V} unioned with BRel (without duplicates!)
   if {BRelHas BRel K V}
   then BRel
   else (K#V)|BRel
   end
end

fun {AsBRel ListOfPairs}
   %% Return the list of pairs without any duplicates.
   case ListOfPairs of
      nil then nil
   [] K#V|Rest then {BRelAdd {AsBRel Rest} K V}
   end
end
   
%%----------------------------------
%% End of provided code.
%%----------------------------------

%%---------------------------------
%% Fill in your code for each of the functions below.
%%---------------------------------

fun {BRelApply BRel Key}
   %% Return a list of all values that BRel associates Key to.
   %% That is, result is { V | Key#V in BRel }
   raise notImplementedYet end %% <-- Replace with your code
end

fun {BRelUnion BRel1 BR2}
   %% Return the union of BRel1 and BR2 (note: without duplicates!)
   %% That is, the result is { K#V | K#V in BRel1 or K#V in BR2 }
   raise notImplementedYet end %% <-- Replace with your code
end

fun {BRelDiff BRel1 BR2}
   %% Return the set of associations in BRel1 without those in BR2.
   %% That is, the result is { K#V | K#V in BRel1 and K#V not in BR2 }.
   raise notImplementedYet end %% <-- Replace with your code
end

fun {BRelCompose BRel1 BR2}
   %% Return the composition of BRel1 and BR2; that is the relation such that
   %% if BRel1 has K#V and BR2 has V#W, then the result has K#W.
   %% That is, the result is { K#W | K#V in Brel1 and V#W in BR2 }.
   raise notImplementedYet end %% <-- Replace with your code
end

fun {ProductBRel Keys Vals}
   %% Assume that Keys and Vals have no duplicates.
   %% Return the product relation of Keys x Vals, which is the relation
   %% { K#V | K in Keys and V in Vals }
   raise notImplementedYet end %% <-- Replace with your code
end
